@@ -1,15 +1,16 @@
 'use strict';
 
-var fs       = require('fs');
-var path     = require('path');
-var expect   = require('expect.js');
-var rimraf   = require('rimraf');
-var mkdirp   = require('mkdirp');
-var which    = require('which');
+var fs = require('fs');
+var path = require('path');
+var expect = require('expect.js');
+var rimraf = require('rimraf');
+var mkdirp = require('mkdirp');
+var which = require('which');
 var buffered = require('./util/buffered');
-var spawn    = require('../');
+var hasBrokenSpawn = require('../lib/hasBrokenSpawn');
+var spawn = require('../');
 
-var isWin    = process.platform === 'win32';
+var isWin = process.platform === 'win32';
 
 // Fix AppVeyor tests because Git bin folder is in PATH and it has a "echo" program there
 if (isWin) {
@@ -25,7 +26,7 @@ describe('cross-spawn', function () {
     var methods = ['spawn', 'sync'];
 
     methods.forEach(function (method) {
-        describe(method, function() {
+        describe(method, function () {
             var originalPath = process.env.PATH;
 
             before(function () {
@@ -36,7 +37,7 @@ describe('cross-spawn', function () {
                 // Give it some time, RIMRAF was giving problems on windows
                 this.timeout(10000);
 
-                rimraf(__dirname + '/tmp', function (err) {
+                rimraf(__dirname + '/tmp', function () {
                     // Ignore errors, RIMRAF was giving problems on windows
                     next(null);
                 });
@@ -70,7 +71,7 @@ describe('cross-spawn', function () {
                 var file = __dirname + '/fixtures/shebang_noenv';
 
                 fs.writeFileSync(file, '#!' + nodejs + '\n\nprocess.stdout.write(\'shebang works!\');', {
-                    mode: parseInt('0777', 8)
+                    mode: parseInt('0777', 8),
                 });
 
                 buffered(method, file, function (err, data, code) {
@@ -94,7 +95,24 @@ describe('cross-spawn', function () {
             it('should support shebang in executables with relative path', function (next) {
                 var executable = './' + path.relative(process.cwd(), __dirname + '/fixtures/shebang');
 
-                fs.writeFileSync(__dirname + '/tmp/shebang', '#!/usr/bin/env node\n\nprocess.stdout.write(\'yeah\');', { mode: parseInt('0777', 8) });
+                fs.writeFileSync(__dirname + '/tmp/shebang', '#!/usr/bin/env node\n\nprocess.stdout.write(\'yeah\');',
+                    { mode: parseInt('0777', 8) });
+                process.env.PATH = path.normalize(__dirname + '/tmp/') + path.delimiter + process.env.PATH;
+
+                buffered(method, executable, function (err, data, code) {
+                    expect(err).to.not.be.ok();
+                    expect(code).to.be(0);
+                    expect(data).to.equal('shebang works!');
+
+                    next();
+                });
+            });
+
+            it('should support shebang in executables with relative path that starts with `..`', function (next) {
+                var executable = '../' + path.basename(process.cwd()) + '/' + path.relative(process.cwd(), __dirname + '/fixtures/shebang');
+
+                fs.writeFileSync(__dirname + '/tmp/shebang', '#!/usr/bin/env node\n\nprocess.stdout.write(\'yeah\');',
+                    { mode: parseInt('0777', 8) });
                 process.env.PATH = path.normalize(__dirname + '/tmp/') + path.delimiter + process.env.PATH;
 
                 buffered(method, executable, function (err, data, code) {
@@ -107,7 +125,8 @@ describe('cross-spawn', function () {
             });
 
             it('should support shebang in executables with extensions', function (next) {
-                fs.writeFileSync(__dirname + '/tmp/shebang_' + method + '.js', '#!/usr/bin/env node\n\nprocess.stdout.write(\'shebang with extension\');', { mode: parseInt('0777', 8) });
+                fs.writeFileSync(__dirname + '/tmp/shebang_' + method + '.js', '#!/usr/bin/env node\n\nprocess.stdout.write(\'shebang with \
+extension\');', { mode: parseInt('0777', 8) });
                 process.env.PATH = path.normalize(__dirname + '/tmp/') + path.delimiter + process.env.PATH;
 
                 buffered(method, __dirname + '/tmp/shebang_' + method + '.js', function (err, data, code) {
@@ -158,12 +177,26 @@ describe('cross-spawn', function () {
                 });
             });
 
+            it('should handle arguments with quotes', function (next) {
+                buffered(method, 'node', [
+                    __dirname + '/fixtures/echo',
+                    '"foo"',
+                    'foo"bar"foo',
+                ], function (err, data, code) {
+                    expect(err).to.not.be.ok();
+                    expect(code).to.be(0);
+                    expect(data).to.equal('"foo"\nfoo"bar"foo');
+
+                    next();
+                });
+            });
+
             it('should handle empty arguments', function (next) {
                 buffered(method, 'node', [
                     __dirname + '/fixtures/echo',
                     'foo',
                     '',
-                    'bar'
+                    'bar',
                 ], function (err, data, code) {
                     expect(err).to.not.be.ok();
                     expect(code).to.be(0);
@@ -172,7 +205,7 @@ describe('cross-spawn', function () {
                     buffered(method, 'echo', [
                         'foo',
                         '',
-                        'bar'
+                        'bar',
                     ], function (err, data, code) {
                         expect(err).to.not.be.ok();
                         expect(code).to.be(0);
@@ -186,7 +219,7 @@ describe('cross-spawn', function () {
             it('should handle non-string arguments', function (next) {
                 buffered(method, 'node', [
                     __dirname + '/fixtures/echo',
-                    1234
+                    1234,
                 ], function (err, data, code) {
                     expect(err).to.not.be.ok();
                     expect(code).to.be(0);
@@ -200,7 +233,7 @@ describe('cross-spawn', function () {
                 buffered(method, 'node', [
                     __dirname + '/fixtures/echo',
                     'I am',
-                    'André Cruz'
+                    'André Cruz',
                 ], function (err, data, code) {
                     expect(err).to.not.be.ok();
                     expect(code).to.be(0);
@@ -215,7 +248,7 @@ describe('cross-spawn', function () {
                     __dirname + '/fixtures/echo',
                     'foo',
                     '\\"',
-                    'bar'
+                    'bar',
                 ], function (err, data, code) {
                     expect(err).to.not.be.ok();
                     expect(code).to.be(0);
@@ -230,7 +263,7 @@ describe('cross-spawn', function () {
                     __dirname + '/fixtures/echo',
                     'foo',
                     'bar\\',
-                    'baz'
+                    'baz',
                 ], function (err, data, code) {
                     expect(err).to.not.be.ok();
                     expect(code).to.be(0);
@@ -255,7 +288,7 @@ describe('cross-spawn', function () {
                     '|;',
                     'foo',
                     ', ',
-                    'foo'
+                    'foo',
                 ], function (err, data, code) {
                     expect(err).to.not.be.ok();
                     expect(code).to.be(0);
@@ -284,7 +317,7 @@ describe('cross-spawn', function () {
                         '|;',
                         'foo',
                         ', ',
-                        'foo'
+                        'foo',
                     ], function (err, data, code) {
                         expect(err).to.not.be.ok();
                         expect(code).to.be(0);
@@ -418,7 +451,7 @@ describe('cross-spawn', function () {
 
                 if (method === 'spawn') {
                     spawned
-                    .on('error', function (err) {
+                    .on('error', function () {
                         spawned.removeAllListeners();
                         clearTimeout(timeout);
                         next(new Error('Should not emit error'));
@@ -448,7 +481,7 @@ describe('cross-spawn', function () {
 
                 if (method === 'spawn') {
                     spawned
-                    .on('error', function (err) {
+                    .on('error', function () {
                         spawned.removeAllListeners();
                         clearTimeout(timeout);
                         next(new Error('Should not emit error'));
@@ -468,6 +501,28 @@ describe('cross-spawn', function () {
                     next();
                 }
             });
+
+            if (isWin) {
+                if (hasBrokenSpawn) {
+                    it('should spawn a shell for a .exe on old Node', function (next) {
+                        buffered(method, __dirname + '/fixtures/win-ppid.js', function (err, data, code) {
+                            expect(err).to.not.be.ok();
+                            expect(code).to.be(0);
+                            expect(data.trim()).to.not.equal('' + process.pid);
+                            next();
+                        });
+                    });
+                } else {
+                    it('should NOT spawn a shell for a .exe', function (next) {
+                        buffered(method, __dirname + '/fixtures/win-ppid.js', function (err, data, code) {
+                            expect(err).to.not.be.ok();
+                            expect(code).to.be(0);
+                            expect(data.trim()).to.equal('' + process.pid);
+                            next();
+                        });
+                    });
+                }
+            }
         });
     });
 });
